@@ -32,9 +32,10 @@
 #ifdef PCAPPLAY
 #include "send_packets.h"
 #endif
-#ifdef RTP_STREAM
 #include "rtpstream.hpp"
-#endif
+#include "jlsrtp.hpp"
+
+#include <stdarg.h>
 
 #ifndef MAX
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
@@ -59,6 +60,18 @@ struct txnInstanceInfo {
     int ackIndex;
 };
 
+typedef enum
+{
+    eNoSession,
+    eOfferReceived,
+    eOfferSent,
+    eOfferRejected,
+    eAnswerReceived,
+    eAnswerSent,
+    eCompleted,
+    eNumSessionStates
+} SessionState;
+
 class call : virtual public task, virtual public listener, public virtual socketowner
 {
 public:
@@ -106,6 +119,9 @@ public:
 private:
     /* This is the core constructor function. */
     void init(scenario * call_scenario, SIPpSocket *socket, struct sockaddr_storage *dest, const char * p_id, int userId, bool ipv6, bool isAutomatic, bool isInitCall);
+
+    bool checkAckCSeq(const char* msg);
+
     /* This this call for initialization? */
     bool initCall;
 
@@ -145,6 +161,8 @@ protected:
     int            last_recv_index;
     char         * last_recv_msg;
 
+    unsigned long int last_recv_invite_cseq;
+
     /* Recv message characteristics when we sent a valid message
      *  (scneario, no retrans) just after a valid reception. This was
      * a cause relationship, so the next time this cookie will be recvd,
@@ -169,9 +187,19 @@ protected:
     play_args_t play_args_v;
 #endif
 
-#ifdef RTP_STREAM
     rtpstream_callinfo_t rtpstream_callinfo;
-#endif
+    JLSRTP _txUACAudio;
+    JLSRTP _rxUACAudio;
+    JLSRTP _txUASAudio;
+    JLSRTP _rxUASAudio;
+    JLSRTP _txUACVideo;
+    JLSRTP _rxUACVideo;
+    JLSRTP _txUASVideo;
+    JLSRTP _rxUASVideo;
+#ifdef USE_TLS
+    char _pref_audio_cs_out[25];
+    char _pref_video_cs_out[25];
+#endif // USE_TLS
 
     /* holds the auth header and if the challenge was 401 or 407 */
     char         * dialog_authentication;
@@ -224,7 +252,8 @@ protected:
         E_AR_TEST_DOESNT_MATCH,
         E_AR_TEST_SHOULDNT_MATCH,
         E_AR_STRCMP_DOESNT_MATCH,
-        E_AR_STRCMP_SHOULDNT_MATCH
+        E_AR_STRCMP_SHOULDNT_MATCH,
+        E_AR_RTPECHO_ERROR
     };
 
     /* Store the last action result to allow  */
@@ -243,12 +272,10 @@ protected:
     bool  rejectCall();
     double get_rhs(CAction *currentAction);
 
-    // P_index use for message index in scenario and ctrl of CRLF
-    // P_index = -2 No ctrl of CRLF
-    // P_index = -1 Add crlf to end of message
-    char* createSendingMessage(SendingMessage *src, int P_index, int *msgLen=NULL);
-    char* createSendingMessage(char * src, int P_index, bool skip_sanity = false);
-    char* createSendingMessage(SendingMessage *src, int P_index, char *msg_buffer, int buflen, int *msgLen=NULL);
+    // P_index use for message index in scenario
+    char* createSendingMessage(SendingMessage* src, int P_index=-1, int *msgLen=NULL);
+    char* createSendingMessage(char* src, int P_index, bool skip_sanity=false);
+    char* createSendingMessage(SendingMessage*src, int P_index, char *msg_buffer, int buflen, int *msgLen=NULL);
 
     // method for the management of unexpected messages
     bool  checkInternalCmd(char* cmd);  // check of specific internal command
@@ -305,23 +332,40 @@ protected:
 
     void get_remote_media_addr(std::string const &msg);
 
-#ifdef RTP_STREAM
+    std::string extract_rtp_remote_addr(const char * message, int &ip_ver, int &audio_port, int &video_port);
+#ifdef USE_TLS
+    int check_audio_ciphersuite_match(SrtpAudioInfoParams &pA);
+    int check_video_ciphersuite_match(SrtpVideoInfoParams &pV);
+    int extract_srtp_remote_info(const char * msg, SrtpAudioInfoParams &pA, SrtpVideoInfoParams &pV);
+#endif // USE_TLS
     void extract_rtp_remote_addr(const char* message);
-#endif
 
     bool lost(int index);
 
+    void setRtpEchoErrors(int value);
+    int getRtpEchoErrors();
+
     void computeStat (CStat::E_Action P_action);
     void computeStat (CStat::E_Action P_action, unsigned long P_value);
     void computeStat (CStat::E_Action P_action, unsigned long P_value, int which);
 
-
     void queue_up(const char* msg);
     char *queued_msg;
 
     int _callDebug(const char *fmt, ...) __attribute__((format(printf, 2, 3)));
     char *debugBuffer;
     int debugLength;
+
+#ifdef USE_TLS
+    FILE* _srtpctxdebugfile;
+    int logSrtpInfo(const char *fmt, ...);
+#endif // USE_TLS
+
+    SessionState _sessionStateCurrent;
+    SessionState _sessionStateOld;
+    void setSessionState(SessionState state);
+    SessionState getSessionStateCurrent();
+    SessionState getSessionStateOld();
 };
 
 
