@@ -25,6 +25,8 @@
 #define __SCENARIO__
 
 #include <map>
+#include <unordered_map>
+
 #include <sys/socket.h>
 #include "actions.hpp"
 #include "variables.hpp"
@@ -189,6 +191,10 @@ public:
     int get_rtd(const char *ptr, bool start);
     int find_var(const char *varName);
 
+    void addRtpTaskThreadID(pthread_t id);
+    void removeRtpTaskThreadID(pthread_t id);
+    std::unordered_map<pthread_t, std::string>& fetchRtpTaskThreadIDs();
+
     CStat *stats;
     AllocVariableTable *allocVars;
 
@@ -220,6 +226,8 @@ private:
 
     bool hidedefault;
     bool last_recv_optional;
+
+    std::unordered_map<pthread_t, std::string> threadIDs;
 };
 
 /* There are external variable containing the current scenario */
