@@ -22,6 +22,7 @@
  */
 
 #include "sipp.hpp"
+#include "fileutil.h"
 #include <assert.h>
 #ifdef PCAPPLAY
 #include "prepare_pcap.h"
@@ -92,7 +93,7 @@ bool CAction::compare(VariableTable *variableTable)
 void CAction::printInfo(char* buf, int len)
 {
     if (M_action == E_AT_ASSIGN_FROM_REGEXP) {
-        if(M_lookingPlace == E_LP_MSG) {
+        if (M_lookingPlace == E_LP_MSG) {
             snprintf(buf, len, "Type[%d] - regexp[%s] where[%s] - checkIt[%d] - checkItInverse[%d] - $%s",
                    M_action,
                    M_regularExpression,
@@ -143,6 +144,10 @@ void CAction::printInfo(char* buf, int len)
         snprintf(buf, len, "Type[%d] - divide varId[%s] %lf", M_action, display_scenario->allocVars->getName(M_varId), M_doubleValue);
     } else if (M_action == E_AT_VAR_TRIM) {
         snprintf(buf, len, "Type[%d] - trim varId[%s]", M_action, display_scenario->allocVars->getName(M_varId));
+    } else if (M_action == E_AT_VAR_URLDECODE) {
+        snprintf(buf, len, "Type[%d] - urldecode varId[%s]", M_action, display_scenario->allocVars->getName(M_varId));
+    } else if (M_action == E_AT_VAR_URLENCODE) {
+        snprintf(buf, len, "Type[%d] - urlencode varId[%s]", M_action, display_scenario->allocVars->getName(M_varId));
     } else if (M_action == E_AT_VAR_TEST) {
         snprintf(buf, len, "Type[%d] - divide varId[%s] varInId[%s] %s %lf", M_action, display_scenario->allocVars->getName(M_varId), display_scenario->allocVars->getName(M_varInId), comparatorToString(M_comp), M_doubleValue);
     } else if (M_action == E_AT_VAR_TO_DOUBLE) {
@@ -153,19 +158,66 @@ void CAction::printInfo(char* buf, int len)
     } else if (M_action == E_AT_PLAY_DTMF) {
         snprintf(buf, len, "Type[%d] - play DTMF digits [%s]", M_action, M_message_str[0]);
 #endif
-
-#ifdef RTP_STREAM
     } else if (M_action == E_AT_RTP_STREAM_PLAY) {
-        snprintf(buf, len, "Type[%d] - rtp_stream playfile file %s loop=%d payload %d bytes per packet=%d ms per packet=%d ticks per packet=%d",
-               M_action, M_rtpstream_actinfo.filename, M_rtpstream_actinfo.loop_count,
-               M_rtpstream_actinfo.payload_type, M_rtpstream_actinfo.bytes_per_packet,
-               M_rtpstream_actinfo.ms_per_packet, M_rtpstream_actinfo.ticks_per_packet);
+        snprintf(buf,
+                 len,
+                 "Type[%d] - rtp_stream playfile file %s pattern_id %d loop=%d payload %d bytes per packet=%d ms per packet=%d ticks per packet=%d",
+                 M_action,
+                 M_rtpstream_actinfo.filename,
+                 M_rtpstream_actinfo.pattern_id,
+                 M_rtpstream_actinfo.loop_count,
+                 M_rtpstream_actinfo.payload_type,
+                 M_rtpstream_actinfo.bytes_per_packet,
+                 M_rtpstream_actinfo.ms_per_packet,
+                 M_rtpstream_actinfo.ticks_per_packet);
     } else if (M_action == E_AT_RTP_STREAM_PAUSE) {
         snprintf(buf, len, "Type[%d] - rtp_stream pause", M_action);
     } else if (M_action == E_AT_RTP_STREAM_RESUME) {
         snprintf(buf, len, "Type[%d] - rtp_stream resume", M_action);
-#endif
-
+    } else if (M_action == E_AT_RTP_STREAM_PLAYAPATTERN) {
+        snprintf(buf,
+                 len,
+                 "Type[%d] - rtp_stream playapattern file %s pattern_id %d loop=%d payload %d bytes per packet=%d ms per packet=%d ticks per packet=%d",
+                 M_action,
+                 M_rtpstream_actinfo.filename,
+                 M_rtpstream_actinfo.pattern_id,
+                 M_rtpstream_actinfo.loop_count,
+                 M_rtpstream_actinfo.payload_type,
+                 M_rtpstream_actinfo.bytes_per_packet,
+                 M_rtpstream_actinfo.ms_per_packet,
+                 M_rtpstream_actinfo.ticks_per_packet);
+    } else if (M_action == E_AT_RTP_STREAM_PAUSEAPATTERN) {
+        snprintf(buf, len, "Type[%d] - rtp_stream pauseapattern", M_action);
+    } else if (M_action == E_AT_RTP_STREAM_RESUMEAPATTERN) {
+        snprintf(buf, len, "Type[%d] - rtp_stream resumeapattern", M_action);
+    } else if (M_action == E_AT_RTP_STREAM_PLAYVPATTERN) {
+        snprintf(buf,
+                 len,
+                 "Type[%d] - rtp_stream playvpattern file %s pattern_id %d loop=%d payload %d bytes per packet=%d ms per packet=%d ticks per packet=%d",
+                 M_action,
+                 M_rtpstream_actinfo.filename,
+                 M_rtpstream_actinfo.pattern_id,
+                 M_rtpstream_actinfo.loop_count,
+                 M_rtpstream_actinfo.payload_type,
+                 M_rtpstream_actinfo.bytes_per_packet,
+                 M_rtpstream_actinfo.ms_per_packet,
+                 M_rtpstream_actinfo.ticks_per_packet);
+    } else if (M_action == E_AT_RTP_STREAM_PAUSEVPATTERN) {
+        snprintf(buf, len, "Type[%d] - rtp_stream pausevpattern", M_action);
+    } else if (M_action == E_AT_RTP_STREAM_RESUMEVPATTERN) {
+        snprintf(buf, len, "Type[%d] - rtp_stream resumevpattern", M_action);
+    } else if (M_action == E_AT_RTP_STREAM_RTPECHO_STARTAUDIO) {
+        snprintf(buf, len, "Type[%d] - rtp_stream rtpecho startaudio", M_action);
+    } else if (M_action == E_AT_RTP_STREAM_RTPECHO_UPDATEAUDIO) {
+        snprintf(buf, len, "Type[%d] - rtp_stream rtpecho updateaudio", M_action);
+    } else if (M_action == E_AT_RTP_STREAM_RTPECHO_STOPAUDIO) {
+        snprintf(buf, len, "Type[%d] - rtp_stream rtpecho stopaudio", M_action);
+    } else if (M_action == E_AT_RTP_STREAM_RTPECHO_STARTVIDEO) {
+        snprintf(buf, len, "Type[%d] - rtp_stream rtpecho startvideo", M_action);
+    } else if (M_action == E_AT_RTP_STREAM_RTPECHO_UPDATEVIDEO) {
+        snprintf(buf, len, "Type[%d] - rtp_stream rtpecho updatevideo", M_action);
+    } else if (M_action == E_AT_RTP_STREAM_RTPECHO_STOPVIDEO) {
+        snprintf(buf, len, "Type[%d] - rtp_stream rtpecho stopvideo", M_action);
     } else {
         snprintf(buf, len, "Type[%d] - unknown action type ... ", M_action);
     }
@@ -242,14 +294,20 @@ char*          CAction::getStringValue()
     return(M_stringValue);
 }
 #ifdef PCAPPLAY
-pcap_pkts  *   CAction::getPcapPkts()
+pcap_pkts *CAction::getPcapPkts()
 {
     return(M_pcapArgs);
 }
 #endif
-#ifdef RTP_STREAM
-rtpstream_actinfo_t *CAction::getRTPStreamActInfo() { return (&M_rtpstream_actinfo); }
-#endif
+rtpecho_actinfo_t* CAction::getRTPEchoActInfo()
+{
+    return &M_rtpecho_actinfo;
+}
+
+rtpstream_actinfo_t* CAction::getRTPStreamActInfo()
+{
+    return &M_rtpstream_actinfo;
+}
 
 void CAction::setActionType   (CAction::T_ActionType   P_value)
 {
@@ -338,7 +396,7 @@ int*  CAction::getSubVarId()
 void CAction::setNbSubVarId (int            P_value)
 {
     M_maxNbSubVarId        = P_value;
-    if(M_subVarId != NULL) {
+    if (M_subVarId != NULL) {
         delete [] M_subVarId;
         M_subVarId      = NULL;
     }
@@ -353,12 +411,12 @@ int  CAction::getNbSubVarId ()
 
 void CAction::setLookingChar(const char* P_value)
 {
-    if(M_lookingChar != NULL) {
+    if (M_lookingChar != NULL) {
         delete [] M_lookingChar;
         M_lookingChar = NULL;
     }
 
-    if(P_value != NULL) {
+    if (P_value != NULL) {
         M_lookingChar = new char[strlen(P_value)+1];
         strcpy(M_lookingChar, P_value);
     }
@@ -366,14 +424,14 @@ void CAction::setLookingChar(const char* P_value)
 
 void CAction::setMessage(const char* P_value, int n)
 {
-    if(M_message[n] != NULL) {
+    if (M_message[n] != NULL) {
         delete M_message[n];
         M_message[n] = NULL;
     }
     free(M_message_str[n]);
     M_message_str[n] = NULL;
 
-    if(P_value != NULL) {
+    if (P_value != NULL) {
         M_message_str[n] = strdup(P_value);
         M_message[n] = new SendingMessage(M_scenario, P_value, true /* skip sanity */);
     }
@@ -388,7 +446,7 @@ void CAction::setRegExp(const char *P_value)
     M_regExpSet = true;
 
     errorCode = regcomp(&M_internalRegExp, P_value, REGCOMP_PARAMS);
-    if(errorCode != 0) {
+    if (errorCode != 0) {
         char buffer[MAX_HEADER_LEN];
         regerror(errorCode, &M_internalRegExp, buffer, sizeof(buffer));
         ERROR("recomp error : regular expression '%s' - error '%s'", M_regularExpression, buffer);
@@ -421,19 +479,18 @@ int CAction::executeRegExp(const char* P_string, VariableTable *P_callVarTable)
     memset((void*)pmatch, 0, sizeof(regmatch_t)*10);
 
     error = regexec(&M_internalRegExp, P_string, 10, pmatch, REGEXEC_PARAMS);
-    if ( error == 0) {
+    if (error == 0) {
         CCallVariable* L_callVar = P_callVarTable->getVar(getVarId());
 
-        for(int i = 0; i <= getNbSubVarId(); i++) {
-            if(pmatch[i].rm_eo != -1) {
+        for (int i = 0; i <= getNbSubVarId(); i++) {
+            if (pmatch[i].rm_eo != -1) {
                 setSubString(&result, P_string, pmatch[i].rm_so, pmatch[i].rm_eo);
                 L_callVar->setMatchingValue(result);
                 nbOfMatch++;
             }
-
-            if (i == getNbSubVarId())
-                break ;
-
+            if (i == getNbSubVarId()) {
+                break;
+            }
             L_callVar = P_callVarTable->getVar(getSubVarId(i));
         }
     }
@@ -444,7 +501,7 @@ void CAction::setSubString(char** P_target, const char* P_source, int P_start, i
 {
     int sizeOf;
 
-    if(P_source != NULL) {
+    if (P_source != NULL) {
         sizeOf = P_stop - P_start;
         (*P_target) = new char[sizeOf + 1];
 
@@ -462,12 +519,12 @@ void CAction::setSubString(char** P_target, const char* P_source, int P_start, i
 #ifdef PCAPPLAY
 void CAction::setPcapArgs (pcap_pkts  *  P_value)
 {
-    if(M_pcapArgs != NULL) {
+    if (M_pcapArgs != NULL) {
         free(M_pcapArgs);
         M_pcapArgs = NULL;
     }
 
-    if(P_value != NULL) {
+    if (P_value != NULL) {
         M_pcapArgs = (pcap_pkts *)malloc(sizeof(*M_pcapArgs));
         memcpy(M_pcapArgs, P_value, sizeof(*M_pcapArgs));
     }
@@ -475,12 +532,12 @@ void CAction::setPcapArgs (pcap_pkts  *  P_value)
 
 void CAction::setPcapArgs(const char* P_value)
 {
-    if(M_pcapArgs != NULL) {
+    if (M_pcapArgs != NULL) {
         free(M_pcapArgs);
         M_pcapArgs = NULL;
     }
 
-    if(P_value != NULL) {
+    if (P_value != NULL) {
         M_pcapArgs = (pcap_pkts *) malloc(sizeof(*M_pcapArgs));
         if (parse_play_args(P_value, M_pcapArgs) == -1) {
             ERROR("Play pcap error");
@@ -492,91 +549,345 @@ void CAction::setPcapArgs(const char* P_value)
 }
 #endif
 
-#ifdef RTP_STREAM
-void CAction::setRTPStreamActInfo(const char* P_value)
+void CAction::setRTPEchoActInfo(const char* P_value)
 {
     char* param_str;
     char* next_comma;
+    char actionstring[RTPECHO_MAX_FILENAMELEN];
 
-    if (strlen(P_value) >= sizeof(M_rtpstream_actinfo.filename)) {
-        ERROR("Filename %s is too long, maximum supported length %zu", P_value,
-              sizeof(M_rtpstream_actinfo.filename) - 1);
+    if (strlen(P_value) >= 512) {
+        ERROR("RTPEcho keyword %s is too long -- maximum supported length %u\n", P_value, 512);
     }
-    strcpy(M_rtpstream_actinfo.filename, P_value);
-    param_str = strchr(M_rtpstream_actinfo.filename, ',');
+
+    memset(actionstring, 0, sizeof(actionstring));
+
+    // Initialize M_rtpecho_actinfo struct members
+    M_rtpecho_actinfo.payload_type = rtp_default_payload;
+    memset(M_rtpecho_actinfo.payload_name, 0, sizeof(M_rtpecho_actinfo.payload_name));
+    M_rtpecho_actinfo.bytes_per_packet = 0;
+    M_rtpecho_actinfo.audio_active = 0;
+    M_rtpecho_actinfo.video_active = 0;
+
+    strcpy (actionstring,P_value);
+    param_str = strchr(actionstring,',');
     next_comma = NULL;
 
-    M_rtpstream_actinfo.loop_count = 1;
+    // Comma found for payload_type parameter
     if (param_str) {
-        /* we have a loop count parameter */
-        *(param_str++) = 0;
-        next_comma= strchr(param_str, ',');
+        *(param_str++) = 0; // skip comma
+
+        /* we have a payload type parameter */
+        next_comma = strchr (param_str, ',');
         if (next_comma) {
             *(next_comma++) = 0;
         }
-        M_rtpstream_actinfo.loop_count = atoi(param_str);
+        M_rtpecho_actinfo.payload_type = atoi(param_str);
         param_str = next_comma;
     }
 
-    M_rtpstream_actinfo.payload_type= rtp_default_payload;
+    // Comma found for payload_name parameter
+    if (param_str) {
+        /* we have a payload_name parameter */
+        next_comma = strchr(param_str, ',');
+        if (next_comma) {
+            *(next_comma++) = 0;
+        }
+        strcpy(M_rtpecho_actinfo.payload_name, param_str);
+        param_str = next_comma;
+    }
+
+    if (!strcmp(M_rtpecho_actinfo.payload_name, "")) {
+        static const std::map<int, const char *> default_parameters{
+            {0, "PCMU/8000"}, {8, "PCMA/8000"},
+            {9, "G722/8000"}, {18, "G729/8000"}};
+        auto const it = default_parameters.find(M_rtpecho_actinfo.payload_type);
+        if (it != default_parameters.end()) {
+            strncat(M_rtpecho_actinfo.payload_name, it->second,
+                    sizeof(M_rtpecho_actinfo.payload_name) - 1);
+        } else {
+            ERROR("Missing mandatory payload_name parameter in rtp_echo action");
+        }
+    }
+
+    /* Setup based on what we know of payload types */
+    switch (M_rtpecho_actinfo.payload_type) {
+    case 0:
+        if (!strcmp(M_rtpecho_actinfo.payload_name, "PCMU/8000")) {
+            M_rtpecho_actinfo.bytes_per_packet = 160;
+            M_rtpecho_actinfo.audio_active = 1;
+        }
+        break;
+    case 8:
+        if (!strcmp(M_rtpecho_actinfo.payload_name, "PCMA/8000")) {
+            M_rtpecho_actinfo.bytes_per_packet = 160;
+            M_rtpecho_actinfo.audio_active = 1;
+        }
+        break;
+    case 9:
+        if (!strcmp(M_rtpecho_actinfo.payload_name, "G722/8000")) {
+            M_rtpecho_actinfo.bytes_per_packet = 160;
+            M_rtpecho_actinfo.audio_active = 1;
+        }
+        break;
+    case 18:
+        if (!strcmp(M_rtpecho_actinfo.payload_name, "G729/8000")) {
+            M_rtpecho_actinfo.bytes_per_packet = 20;
+            M_rtpecho_actinfo.audio_active = 1;
+        }
+        break;
+    default:
+        if (M_rtpecho_actinfo.payload_type >= 0 &&
+                M_rtpecho_actinfo.payload_type <= 95) {
+            M_rtpecho_actinfo.bytes_per_packet = -1;
+            M_rtpecho_actinfo.audio_active = 0;
+            M_rtpecho_actinfo.video_active = 0;
+            ERROR("Unknown static rtp payload type %d - cannot set playback parameters\n",
+                M_rtpecho_actinfo.payload_type);
+        } else if (M_rtpecho_actinfo.payload_type >= 96 &&
+                M_rtpecho_actinfo.payload_type <= 127) {
+            if (!strcmp(M_rtpecho_actinfo.payload_name, "H264/90000")) {
+                M_rtpecho_actinfo.bytes_per_packet = 1280;  // ARBITRARY H264 PACKET SIZE
+                M_rtpecho_actinfo.video_active = 1;
+            } else if (!strcmp(M_rtpecho_actinfo.payload_name, "iLBC/8000")) {
+                M_rtpecho_actinfo.bytes_per_packet = 50;
+                M_rtpecho_actinfo.audio_active = 1;
+            } else {
+                M_rtpecho_actinfo.bytes_per_packet = -1;
+                M_rtpecho_actinfo.audio_active = 0;
+                M_rtpecho_actinfo.video_active = 0;
+                ERROR("Unknown dynamic rtp payload type %d - cannot set playback parameters\n",M_rtpecho_actinfo.payload_type);
+            }
+        } else {
+            ERROR("Invalid rtp payload type %d - cannot set playback parameters\n",M_rtpecho_actinfo.payload_type);
+        }
+    }
+}
+
+void CAction::setRTPEchoActInfo(rtpecho_actinfo_t *P_value)
+{
+    /* At this stage the entire rtpecho action info structure can simply be */
+    /* copied. No members need to be individually duplicated/processed. */
+    memcpy(&M_rtpecho_actinfo, P_value, sizeof(M_rtpecho_actinfo));
+}
+
+void CAction::setRTPStreamActInfo(const char *P_value)
+{
+    char* param_str;
+    char* next_comma;
+    int pattern_mode = 0;
+    int stream_type = 0; /* 0: AUDIO / 1: VIDEO */
+    char argument_buf[sizeof(M_rtpstream_actinfo.filename)];
+    const char* found_file;
+
+    if (strlen(P_value) >= sizeof(argument_buf)) {
+        ERROR("Filename/Pattern keyword %s is too long -- maximum supported length %zu",
+            P_value, sizeof(argument_buf) - 1);
+    }
+
+    // Initialize M_rtpstream_actinfo struct members
+    M_rtpstream_actinfo.filename[0] = '\0';
+    M_rtpstream_actinfo.pattern_id = -1;
+    M_rtpstream_actinfo.loop_count = -1;
+    M_rtpstream_actinfo.bytes_per_packet = 0;
+    M_rtpstream_actinfo.ms_per_packet = 0;
+    M_rtpstream_actinfo.ticks_per_packet = 0;
+    M_rtpstream_actinfo.payload_type = 0;
+    M_rtpstream_actinfo.payload_name[0] = '\0';
+    M_rtpstream_actinfo.audio_active = 0;
+    M_rtpstream_actinfo.video_active = 0;
+
+    if (!strncmp(P_value, "apattern", 8) || !strncmp(P_value, "vpattern", 8)) {
+        pattern_mode = 1;
+    }
+
+    // Extract filename
+    strcpy(argument_buf, P_value);
+    if ((param_str = strchr(argument_buf, ','))) {
+        *param_str++ = '\0';
+    }
+
+    // Lookup best file match
+    if (pattern_mode == 0) {
+        found_file = find_file(argument_buf);
+        if (found_file) {
+            if (strlen(found_file) >= sizeof(M_rtpstream_actinfo.filename)) {
+                ERROR("Filename/Pattern keyword %s is too long -- maximum supported length %zu",
+                    found_file, sizeof(M_rtpstream_actinfo.filename) - 1);
+            }
+            strcpy(M_rtpstream_actinfo.filename, found_file);
+            free(const_cast<char*>(found_file));
+        }
+    }
+
+    // Set default values of pattern_id/loop_count depending on whether we are in PATTERN or FILE mode
+    if (pattern_mode) {
+        M_rtpstream_actinfo.pattern_id = 1;
+        M_rtpstream_actinfo.loop_count = -1;
+    } else {
+        M_rtpstream_actinfo.pattern_id = -1;
+        M_rtpstream_actinfo.loop_count = 1;
+    }
+
+    // Comma found for loop_count (FILE MODE) or pattern_id (PATTERN MODE)
+    if (param_str) {
+        /* we have a loop count parameter (FILE MODE) or pattern id parameter (PATTERN MODE) */
+        next_comma = strchr(param_str, ',');
+        if (next_comma) {
+            *(next_comma++) = 0;
+        }
+
+        if (pattern_mode) {
+            M_rtpstream_actinfo.pattern_id = atoi(param_str);
+        } else {
+            M_rtpstream_actinfo.loop_count = atoi(param_str);
+        }
+        param_str = next_comma;
+    }
+
+    // Set default RTP payload type value
+    M_rtpstream_actinfo.payload_type = rtp_default_payload;
+
+    // Comma found for payload_type parameter
     if (param_str) {
         /* we have a payload type parameter */
-        next_comma= strchr (param_str,',');
-        if (next_comma) {
-            *(next_comma++)= 0;
+        next_comma = strchr(param_str, ',');
+        if (next_comma)
+        {
+            *(next_comma++) = 0;
         }
-        M_rtpstream_actinfo.payload_type= atoi(param_str);
+        M_rtpstream_actinfo.payload_type = atoi(param_str);
+        param_str = next_comma;
     }
 
+    // Comma found for payload_name parameter
+    if (param_str) {
+        /* we have a payload_name parameter */
+        next_comma = strchr(param_str, ',');
+        if (next_comma) {
+            *(next_comma++) = 0;
+        }
+        strcpy(M_rtpstream_actinfo.payload_name, param_str);
+        param_str = next_comma;
+    }
+
+    if (!strcmp(M_rtpstream_actinfo.payload_name, "")) {
+        static const std::map<int, const char *> default_parameters {
+            {0, "PCMU/8000"}, {8, "PCMA/8000"},
+            {9, "G722/8000"}, {18, "G729/8000"}};
+        auto const it = default_parameters.find(M_rtpstream_actinfo.payload_type);
+        if (it != default_parameters.end())
+        {
+            strncat(M_rtpstream_actinfo.payload_name, it->second,
+                    sizeof(M_rtpstream_actinfo.payload_name) - 1);
+        } else {
+            ERROR("Missing mandatory payload_name parameter in rtp_stream action");
+        }
+    }
+
+    // JLTAG NOTE:
+    // Reworked code from commit
+    // "f3b6fe8cd60dd3d1dcca0ab373dc9d289f123f43" which violated the RFC
+    // standards for payload_type IDs by using dynamic payload type 98
+    // incorrectly as a static one for the iLBC audio codec which rather
+    // uses dynamic payload types and should rather be determined by
+    // payload_name (0-95 is the payload_type range reserved for static
+    // codecs, 96-127 is the payload_type range is reserved for dynamic
+    // codecs)
+
     /* Setup based on what we know of payload types */
     switch (M_rtpstream_actinfo.payload_type) {
     case 0:
-        M_rtpstream_actinfo.ms_per_packet = 20;
-        M_rtpstream_actinfo.bytes_per_packet = 160;
-        M_rtpstream_actinfo.ticks_per_packet = 160;
+        if (!strcmp(M_rtpstream_actinfo.payload_name, "PCMU/8000")) {
+            M_rtpstream_actinfo.ms_per_packet = 20;
+            M_rtpstream_actinfo.bytes_per_packet = 160;
+            M_rtpstream_actinfo.ticks_per_packet = 160;
+            M_rtpstream_actinfo.audio_active = 1;
+            stream_type = 0;
+        }
         break;
     case 8:
-        M_rtpstream_actinfo.ms_per_packet = 20;
-        M_rtpstream_actinfo.bytes_per_packet = 160;
-        M_rtpstream_actinfo.ticks_per_packet = 160;
+        if (!strcmp(M_rtpstream_actinfo.payload_name, "PCMA/8000")) {
+            M_rtpstream_actinfo.ms_per_packet = 20;
+            M_rtpstream_actinfo.bytes_per_packet = 160;
+            M_rtpstream_actinfo.ticks_per_packet = 160;
+            M_rtpstream_actinfo.audio_active = 1;
+            stream_type = 0;
+        }
         break;
     case 9:
-        M_rtpstream_actinfo.ms_per_packet = 20;
-        M_rtpstream_actinfo.bytes_per_packet = 160;
-        M_rtpstream_actinfo.ticks_per_packet = 160;
+        if (!strcmp(M_rtpstream_actinfo.payload_name, "G722/8000")) {
+            M_rtpstream_actinfo.ms_per_packet = 20;
+            M_rtpstream_actinfo.bytes_per_packet = 160;
+            M_rtpstream_actinfo.ticks_per_packet = 160;
+            M_rtpstream_actinfo.audio_active = 1;
+            stream_type = 0;
+        }
         break;
     case 18:
-        M_rtpstream_actinfo.ms_per_packet = 20;
-        M_rtpstream_actinfo.bytes_per_packet = 20;
-        M_rtpstream_actinfo.ticks_per_packet = 160;
-        break;
-    case 98:
-        M_rtpstream_actinfo.ms_per_packet = 30;
-        M_rtpstream_actinfo.bytes_per_packet = 50;
-        M_rtpstream_actinfo.ticks_per_packet = 240;
+        if (!strcmp(M_rtpstream_actinfo.payload_name, "G729/8000")) {
+            M_rtpstream_actinfo.ms_per_packet = 20;
+            M_rtpstream_actinfo.bytes_per_packet = 20;
+            M_rtpstream_actinfo.ticks_per_packet = 160;
+            M_rtpstream_actinfo.audio_active = 1;
+            stream_type = 0;
+        }
         break;
     default:
-        M_rtpstream_actinfo.ms_per_packet= -1;
-        M_rtpstream_actinfo.bytes_per_packet= -1;
-        M_rtpstream_actinfo.ticks_per_packet= -1;
-        ERROR("Unknown rtp payload type %d - cannot set playback parameters",
-              M_rtpstream_actinfo.payload_type);
+        if ((M_rtpstream_actinfo.payload_type >= 0) &&
+                (M_rtpstream_actinfo.payload_type <= 95)) {
+            M_rtpstream_actinfo.ms_per_packet = -1;
+            M_rtpstream_actinfo.bytes_per_packet = -1;
+            M_rtpstream_actinfo.ticks_per_packet = -1;
+            M_rtpstream_actinfo.audio_active = 0;
+            M_rtpstream_actinfo.video_active = 0;
+            ERROR("Unknown static rtp payload type %d - cannot set playback parameters\n",
+                M_rtpstream_actinfo.payload_type);
+        } else if (M_rtpstream_actinfo.payload_type >= 96 &&
+                M_rtpstream_actinfo.payload_type <= 127) {
+            if (!strcmp(M_rtpstream_actinfo.payload_name, "H264/90000")) {
+                M_rtpstream_actinfo.ms_per_packet = 160;   // ARBITRARY H264 PACKET TIME
+                M_rtpstream_actinfo.bytes_per_packet = 1280;  // ARBITRARY H264 PACKET SIZE
+                M_rtpstream_actinfo.ticks_per_packet = 1280;  // ARBITRARY H264 PACKET TICKS
+                M_rtpstream_actinfo.video_active = 1;
+                stream_type = 1;
+            } else if (!strcmp(M_rtpstream_actinfo.payload_name, "iLBC/8000")) {
+                M_rtpstream_actinfo.ms_per_packet = 30;
+                M_rtpstream_actinfo.bytes_per_packet = 50;
+                M_rtpstream_actinfo.ticks_per_packet = 240;
+                M_rtpstream_actinfo.audio_active = 1;
+                stream_type = 0;
+            } else  {
+                M_rtpstream_actinfo.ms_per_packet = -1;
+                M_rtpstream_actinfo.bytes_per_packet = -1;
+                M_rtpstream_actinfo.ticks_per_packet = -1;
+                M_rtpstream_actinfo.audio_active = 0;
+                M_rtpstream_actinfo.video_active = 0;
+                ERROR("Unknown dynamic rtp payload type %d - cannot set playback parameters\n",
+                    M_rtpstream_actinfo.payload_type);
+            }
+        } else {
+            ERROR("Invalid rtp payload type %d - cannot set playback parameters\n",
+                M_rtpstream_actinfo.payload_type);
+        }
         break;
     }
 
-    if (rtpstream_cache_file(M_rtpstream_actinfo.filename) < 0) {
-        ERROR("Cannot read/cache rtpstream file %s",
-              M_rtpstream_actinfo.filename);
+    if (rtpstream_cache_file(
+            M_rtpstream_actinfo.filename,
+            pattern_mode /* 0: FILE - 1: PATTERN */,
+            M_rtpstream_actinfo.pattern_id,
+            M_rtpstream_actinfo.bytes_per_packet,
+            stream_type) < 0) {
+        ERROR("Cannot read/cache rtpstream file %s", M_rtpstream_actinfo.filename);
     }
 }
 
 void CAction::setRTPStreamActInfo(rtpstream_actinfo_t *P_value)
 {
     /* At this stage the entire rtpstream action info structure can simply be */
-    /* copied. No members need to be individually duplicated/processed.       */
-    memcpy(&M_rtpstream_actinfo,P_value, sizeof(M_rtpstream_actinfo));
+    /* copied. No members need to be individually duplicated/processed. */
+    memcpy(&M_rtpstream_actinfo, P_value, sizeof(M_rtpstream_actinfo));
 }
-#endif
 
 void CAction::setScenario(scenario *     P_scenario)
 {
@@ -616,9 +927,8 @@ void CAction::setAction(CAction P_action)
 #ifdef PCAPPLAY
     setPcapArgs     ( P_action.M_pcapArgs        );
 #endif
-#ifdef RTP_STREAM
+    setRTPEchoActInfo(&(P_action.M_rtpecho_actinfo));
     setRTPStreamActInfo(&(P_action.M_rtpstream_actinfo));
-#endif
 }
 
 CAction::CAction(scenario *scenario)
@@ -651,9 +961,8 @@ CAction::CAction(scenario *scenario)
     M_pcapArgs     = NULL;
 #endif
 
-#ifdef RTP_STREAM
+    memset(&M_rtpecho_actinfo, 0, sizeof(M_rtpecho_actinfo));
     memset(&M_rtpstream_actinfo, 0, sizeof(M_rtpstream_actinfo));
-#endif
 
     M_scenario     = scenario;
     M_regExpSet    = false;
@@ -662,19 +971,19 @@ CAction::CAction(scenario *scenario)
 
 CAction::~CAction()
 {
-    if(M_lookingChar != NULL) {
+    if (M_lookingChar != NULL) {
         delete [] M_lookingChar;
         M_lookingChar = NULL;
     }
     for (int i = 0; i < MAX_ACTION_MESSAGE; i++) {
-        if(M_message[i] != NULL) {
+        if (M_message[i] != NULL) {
             delete M_message[i];
             M_message[i] = NULL;
         }
         free(M_message_str[i]);
         M_message_str[i] = NULL;
     }
-    if(M_subVarId != NULL) {
+    if (M_subVarId != NULL) {
         delete [] M_subVarId;
         M_subVarId      = NULL;
     }
@@ -699,7 +1008,7 @@ CAction::~CAction()
 void CActions::printInfo()
 {
     printf("Action Size = [%d]\n", M_nbAction);
-    for(int i=0; i<M_nbAction; i++) {
+    for (int i = 0; i < M_nbAction; i++) {
         printf("actionlist[%d] : \n", i);
         char buf[80];
         M_actionList[i]->printInfo(buf, 80);
@@ -740,7 +1049,7 @@ void CActions::setAction(CAction *P_action)
 
 CAction* CActions::getAction(int i)
 {
-    if(i < M_nbAction) {
+    if (i < M_nbAction) {
         return(M_actionList[i]);
     } else
         return(NULL);
