@@ -58,14 +58,42 @@ struct KeywordMap SimpleKeywords[] = {
     {"local_port", E_Message_Local_Port },
     {"server_ip", E_Message_Server_IP },
     {"media_ip", E_Message_Media_IP },
-#ifdef PCAPPLAY
-    {"auto_media_port", E_Message_Auto_Media_Port },
-#endif
-#ifdef RTP_STREAM
-    /* Legacy since 3.6-dev. Actually uses media_port. */
     {"rtpstream_audio_port", E_Message_RTPStream_Audio_Port },
     {"rtpstream_video_port", E_Message_RTPStream_Video_Port },
-#endif
+#ifdef USE_TLS
+    {"cryptotag1audio", E_Message_CryptoTag1Audio },
+    {"cryptotag2audio", E_Message_CryptoTag2Audio },
+    {"cryptosuiteaescm128sha1801audio", E_Message_CryptoSuiteAesCm128Sha1801Audio },
+    {"cryptosuiteaescm128sha1802audio", E_Message_CryptoSuiteAesCm128Sha1802Audio },
+    {"cryptosuiteaescm128sha1321audio", E_Message_CryptoSuiteAesCm128Sha1321Audio },
+    {"cryptosuiteaescm128sha1322audio", E_Message_CryptoSuiteAesCm128Sha1322Audio },
+    {"cryptokeyparams1audio", E_Message_CryptoKeyParams1Audio },
+    {"cryptokeyparams2audio", E_Message_CryptoKeyParams2Audio },
+    {"cryptotag1video", E_Message_CryptoTag1Video },
+    {"cryptotag2video", E_Message_CryptoTag2Video },
+    {"cryptosuiteaescm128sha1801video", E_Message_CryptoSuiteAesCm128Sha1801Video },
+    {"cryptosuiteaescm128sha1802video", E_Message_CryptoSuiteAesCm128Sha1802Video },
+    {"cryptosuiteaescm128sha1321video", E_Message_CryptoSuiteAesCm128Sha1321Video },
+    {"cryptosuiteaescm128sha1322video", E_Message_CryptoSuiteAesCm128Sha1322Video },
+    {"cryptokeyparams1video", E_Message_CryptoKeyParams1Video },
+    {"cryptokeyparams2video", E_Message_CryptoKeyParams2Video },
+    {"cryptosuitenullsha1801audio" , E_Message_CryptoSuiteNullSha1801Audio },
+    {"cryptosuitenullsha1802audio" , E_Message_CryptoSuiteNullSha1802Audio },
+    {"cryptosuitenullsha1321audio" , E_Message_CryptoSuiteNullSha1321Audio },
+    {"cryptosuitenullsha1322audio" , E_Message_CryptoSuiteNullSha1322Audio },
+    {"cryptosuitenullsha1801video" , E_Message_CryptoSuiteNullSha1801Video },
+    {"cryptosuitenullsha1802video" , E_Message_CryptoSuiteNullSha1802Video },
+    {"cryptosuitenullsha1321video" , E_Message_CryptoSuiteNullSha1321Video },
+    {"cryptosuitenullsha1322video" , E_Message_CryptoSuiteNullSha1322Video },
+    {"ueaescm128sha1801audio" , E_Message_UEAesCm128Sha1801Audio },
+    {"ueaescm128sha1802audio" , E_Message_UEAesCm128Sha1802Audio },
+    {"ueaescm128sha1321audio" , E_Message_UEAesCm128Sha1321Audio },
+    {"ueaescm128sha1322audio" , E_Message_UEAesCm128Sha1322Audio },
+    {"ueaescm128sha1801video" , E_Message_UEAesCm128Sha1801Video },
+    {"ueaescm128sha1802video" , E_Message_UEAesCm128Sha1802Video },
+    {"ueaescm128sha1321video" , E_Message_UEAesCm128Sha1321Video },
+    {"ueaescm128sha1322video" , E_Message_UEAesCm128Sha1322Video },
+#endif // USE_TLS
     {"media_port", E_Message_Media_Port },
     {"media_ip_type", E_Message_Media_IP_Type },
     {"call_number", E_Message_Call_Number },
@@ -319,15 +347,8 @@ SendingMessage::SendingMessage(scenario* msg_scenario, const char* const_src, bo
                 newcomp->literalLen = strlen(newcomp->literal);
             } else if(!strncmp(keyword, "authentication", strlen("authentication"))) {
                 parseAuthenticationKeyword(msg_scenario, newcomp, keyword);
-            }
-#ifndef PCAPPLAY
-            else if(!strcmp(keyword, "auto_media_port")) {
-                ERROR("The %s keyword requires PCAPPLAY", keyword);
-            }
-#endif
-            else {
+            } else {
                 // scan for the generic parameters - must be last test
-
                 int i = 0;
                 while (generic[i]) {
                     char *msg1 = *generic[i];
