@@ -48,6 +48,7 @@
 #include <pthread.h>
 
 #include "defines.h"
+#include "fileutil.h"
 #include "send_packets.h"
 #include "prepare_pcap.h"
 #include "config.h"
@@ -59,7 +60,6 @@
 #define uh_dport dest
 #endif
 
-extern char* scenario_path;
 extern volatile unsigned long rtp_pckts_pcap;
 extern volatile unsigned long rtp_bytes_pcap;
 extern bool media_ip_is_ipv6;
@@ -93,27 +93,6 @@ float2timer(float time, struct timeval *tvp)
     tvp->tv_usec = n * 100000;
 }
 
-static char* find_file(const char* filename)
-{
-    char *fullpath;
-    if (filename[0] == '/' || !*scenario_path) {
-        return strdup(filename);
-    }
-
-    fullpath = malloc(MAX_PATH);
-    snprintf(fullpath, MAX_PATH, "%s%s", scenario_path, filename);
-
-    if (access(fullpath, R_OK) < 0) {
-        free(fullpath);
-        WARNING("SIPp now prefers looking for pcap files next to the scenario. "
-                "%s couldn't be found next to the scenario, falling back to "
-                "using the current working directory", filename);
-        return strdup(filename);
-    }
-
-    return fullpath;
-}
-
 int parse_play_args(const char* filename, pcap_pkts* pkts)
 {
     pkts->file = find_file(filename);
