@@ -37,10 +37,6 @@
  *           Walter Doekes
  */
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE /* needed for strcasestr on cygwin */
-#endif
-
 #include <stdlib.h>
 #include <string.h>
 
@@ -71,6 +67,10 @@ static const char* internal_hdrchr(const char* ptr, const char needle);
  * broken, at the ASCIIZ NUL. */
 static const char* internal_hdrend(const char* ptr);
 
+static const char* internal_compact_header_name(const char* name);
+static char* internal_match_header(char* message, const char* hdr, const char* compact_hdr);
+
+
 /*************************** Mini SIP parser (externals) ***************/
 
 char* get_peer_tag(const char* msg)
@@ -123,12 +123,12 @@ char* get_header(const char* message, const char* name, bool content)
     /* non reentrant. consider accepting char buffer as param */
     static char last_header[MAX_HEADER_LEN * 10];
     const char *cptr;
-    char *src, *src_orig, *dest, *start, *ptr;
-    /* Are we searching for a short form header? */
-    bool short_form = false;
+    char *src, *src_copy, *dest, *start, *ptr;
     bool first_time = true;
     char header_with_newline[MAX_HEADER_LEN + 1];
 
+    const char* compact_header_with_newline;
+
     /* returns empty string in case of error */
     last_header[0] = '\0';
 
@@ -142,106 +142,93 @@ char* get_header(const char* message, const char* name, bool content)
         return last_header;
     }
 
+    snprintf(header_with_newline, MAX_HEADER_LEN, "\n%s", name);
+    compact_header_with_newline = internal_compact_header_name(name);
+
     /* find end of SIP headers - perform search only until that */
     cptr = strstr(message, "\r\n\r\n");
     if (!cptr) {
-        src_orig = strdup(message);
-    } else if ((src_orig = (char*)malloc(cptr - message + 1))) {
-        src_orig[cptr - message] = '\0';
-        memcpy(src_orig, message, cptr - message);
+        src_copy = strdup(message);
+    } else if ((src_copy = (char*)malloc(cptr - message + 1))) {
+        src_copy[cptr - message] = '\0';
+        memcpy(src_copy, message, cptr - message);
     }
-    if (!src_orig) {
+    if (!src_copy) {
         ERROR("Out of memory");
         return last_header;
     }
 
-    do {
-        /* We want to start from the beginning of the message each time
-         * through this loop, because we may be searching for a short form. */
-        src = src_orig;
+    src = src_copy;
+    dest = last_header;
 
-        snprintf(header_with_newline, MAX_HEADER_LEN, "\n%s", name);
-        dest = last_header;
+    while ((src = internal_match_header(
+            src, header_with_newline, compact_header_with_newline))) {
+        if (!content && first_time) {
+            // Add the name to the string;
+            dest += sprintf(dest, "%s", name);
+            first_time = false;
+        }
 
-        while ((src = strcasestr(src, header_with_newline))) {
-            if (content || !first_time) {
-                /* Just want the header's content, so skip over the header
-                 * and newline */
-                src += strlen(name) + 1;
-                /* Skip over leading spaces. */
-                while (*src == ' ') {
-                    src++;
-                }
-            } else {
-                /* Just skip the newline */
+        if (content || !first_time) {
+            /* Just want the header's content, so skip over the header
+             * and newline */
+
+            /* Skip over header */
+            while (*src != ':') {
                 src++;
             }
-            first_time = false;
-            ptr = strchr(src, '\n');
+            src++;
 
-            /* Multiline headers always begin with a tab or a space
-             * on the subsequent lines. Skip those lines. */
-            while (ptr && (*(ptr+1) == ' ' || *(ptr+1) == '\t')) {
-                ptr = strchr(ptr + 1, '\n');
+            /* Skip over leading spaces. */
+            while (*src == ' ') {
+                src++;
             }
-
-            if (ptr) {
-                *ptr = 0;
-            }
-            // Add ", " when several headers are present
-            if (dest != last_header) {
-                /* Remove trailing whitespaces, tabs, and CRs */
-                while (dest > last_header &&
-                       (*(dest-1) == ' ' ||
-                        *(dest-1) == '\r' ||
-                        *(dest-1) == '\n' ||
-                        *(dest-1) == '\t')) {
-                    *(--dest) = 0;
-                }
-
-                dest += sprintf(dest, ", ");
-            }
-            dest += sprintf(dest, "%s", src);
-            if (ptr) {
-                *ptr = '\n';
-            }
-
+        } else {
+            /* Just skip the newline */
             src++;
         }
-        /* We found the header. */
-        if (dest != last_header) {
-            break;
-        }
-        /* We didn't find the header, even in its short form. */
-        if (short_form) {
-            free(src_orig);
-            return last_header;
+        ptr = strchr(src, '\n');
+
+        /* Multiline headers always begin with a tab or a space
+         * on the subsequent lines. Skip those lines. */
+        while (ptr && (*(ptr+1) == ' ' || *(ptr+1) == '\t')) {
+            ptr = strchr(ptr + 1, '\n');
         }
 
-        /* We should retry with the short form. */
-        short_form = true;
-        if (!strcasecmp(name, "call-id:")) {
-            name = "i:";
-        } else if (!strcasecmp(name, "contact:")) {
-            name = "m:";
-        } else if (!strcasecmp(name, "content-encoding:")) {
-            name = "e:";
-        } else if (!strcasecmp(name, "content-length:")) {
-            name = "l:";
-        } else if (!strcasecmp(name, "content-type:")) {
-            name = "c:";
-        } else if (!strcasecmp(name, "from:")) {
-            name = "f:";
-        } else if (!strcasecmp(name, "to:")) {
-            name = "t:";
-        } else if (!strcasecmp(name, "via:")) {
-            name = "v:";
-        } else {
-            /* There is no short form to try. */
-            free(src_orig);
-            return last_header;
+        if (ptr) {
+            *ptr = 0;
         }
-    } while (1);
+        // Add ", " when several headers are present
+        if (dest != last_header) {
+            /* Remove trailing whitespaces, tabs, and CRs */
+            while (dest > last_header &&
+                   (*(dest-1) == ' ' ||
+                    *(dest-1) == '\r' ||
+                    *(dest-1) == '\n' ||
+                    *(dest-1) == '\t')) {
+                *(--dest) = 0;
+            }
+
+            if (*(dest-1) == ':') {
+                dest += sprintf(dest, " ");
+            } else {
+                dest += sprintf(dest, ", ");
+            }
+        }
+        dest += sprintf(dest, "%s", src);
+
+        if (ptr) {
+            *ptr = '\n';
+        }
+
+        src++;
+    }
+
+    /* No header found? */
+    if (dest == last_header) {
+        free(src_copy);
+        return last_header;
+    }
 
     *(dest--) = 0;
 
@@ -271,10 +258,58 @@ char* get_header(const char* message, const char* name, bool content)
         memmove(ptr, ptr+1, strlen(ptr));
     }
 
-    free(src_orig);
+    free(src_copy);
     return start;
 }
 
+char* internal_match_header(char* message, const char* hdr, const char* compact_hdr) {
+    // Attempt to find the header
+    char* header_match = strcasestr(message, hdr);
+    if (!compact_hdr) {
+        // Exit when there is no compact header to compare to
+        return header_match;
+    }
+
+    char* compact_header_match = strcasestr(message, compact_hdr);
+
+    // Return the other if one is null
+    if (header_match == nullptr) {
+        return compact_header_match;
+    }
+    if (compact_header_match == nullptr) {
+        return header_match;
+    }
+
+    // Value exists return the smaller of the two.
+    if (header_match < compact_header_match) {
+        return header_match;
+    }
+
+    return compact_header_match;
+}
+
+const char* internal_compact_header_name(const char* name)
+{
+    if (!strcasecmp(name, "call-id:")) {
+        return "\ni:";
+    } else if (!strcasecmp(name, "contact:")) {
+        return "\nm:";
+    } else if (!strcasecmp(name, "content-encoding:")) {
+        return "\ne:";
+    } else if (!strcasecmp(name, "content-length:")) {
+        return "\nl:";
+    } else if (!strcasecmp(name, "content-type:")) {
+        return "\nc:";
+    } else if (!strcasecmp(name, "from:")) {
+        return "\nf:";
+    } else if (!strcasecmp(name, "to:")) {
+        return "\nt:";
+    } else if (!strcasecmp(name, "via:")) {
+        return "\nv:";
+    }
+    return NULL;
+}
+
 char* get_first_line(const char* message)
 {
     /* non reentrant. consider accepting char buffer as param */
@@ -574,6 +609,75 @@ a=rtcp:41605\r\n\
     }
 }
 
+TEST(Parser, get_header_mixed_form) {
+    const char* data = "INVITE sip:3136455552@85.12.1.1:5065 SIP/2.0\r\n\
+v: SIP/2.0/UDP 85.55.55.12:6090;branch=z9hG4bK831a.2bb3de85.0\r\n\
+Via: SIP/2.0/UDP 85.55.55.12:5090;branch=z9hG4bK831a.2bb3de85.0\r\n\
+Via:SIP/2.0/UDP 85.55.55.12:5060;branch=z9hG4bK831a.2bb3de87.0\r\n\
+v: SIP/2.0/UDP 85.55.55.12:4060;branch=z9hG4bK831a.2bb3de86.0\r\n\
+v:SIP/2.0/UDP 85.55.55.12:4050;branch=z9hG4bK831a.2bb3de86.0\r\n\
+Record-Route: <sip:85.55.55.12:5090;r2=on;lr>\r\n\
+Record-Route: <sip:10.231.33.44;r2=on;lr>\r\n\
+Record-Route: <sip:10.231.33.77;lr=on>\r\n\
+From: \"3136456666\" <sip:104@sbc.profxxx.xx>;tag=b62e0d72-be14-4d3c-bd6a-b4da593b6b17\r\n\
+To: <sip:3136455552@sbc2.profxxx.xx>\r\n\
+Contact: <sip:12999999999@85.55.55.12;did=a19.a2e590e>\r\n\
+Call-ID: DLGCH_K0IEXzVwYzJiQlwKMGRkMX5GSAxiKmJ+exQADWYsZ2QsFQFb\r\n\
+CSeq: 6476 INVITE\r\n\
+Allow: OPTIONS, REGISTER, SUBSCRIBE, NOTIFY, PUBLISH, INVITE, ACK, BYE, CANCEL, UPDATE, PRACK, MESSAGE, REFER\r\n\
+Supported: 100rel, timer, replaces, norefersub\r\n\
+Session-Expires: 1800\r\n\
+Min-SE: 90\r\n\
+Max-Forwards: 70\r\n\
+Content-Type: application/sdp\r\n\
+Content-Length: 278\r\n\
+\r\n\
+v=0\r\n\
+o=- 592907310 592907310 IN IP4 85.55.55.30\r\n\
+s=Centrex v.1.0\r\n\
+c=IN IP4 85.55.55.30\r\n\
+t=0 0\r\n\
+m=audio 41604 RTP/AVP 8 0 101\r\n\
+a=rtpmap:8 PCMA/8000\r\n\
+a=rtpmap:0 PCMU/8000\r\n\
+a=rtpmap:101 telephone-event/8000\r\n\
+a=fmtp:101 0-16\r\n\
+a=ptime:20\r\n\
+a=maxptime:150\r\n\
+a=sendrecv\r\n\
+a=rtcp:41605\r\n\
+";
+    EXPECT_STREQ("Via: SIP/2.0/UDP 85.55.55.12:6090;branch=z9hG4bK831a.2bb3de85.0, \
+SIP/2.0/UDP 85.55.55.12:5090;branch=z9hG4bK831a.2bb3de85.0, \
+SIP/2.0/UDP 85.55.55.12:5060;branch=z9hG4bK831a.2bb3de87.0, \
+SIP/2.0/UDP 85.55.55.12:4060;branch=z9hG4bK831a.2bb3de86.0, \
+SIP/2.0/UDP 85.55.55.12:4050;branch=z9hG4bK831a.2bb3de86.0", get_header(data, "Via:", false));
+
+    EXPECT_STREQ("SIP/2.0/UDP 85.55.55.12:6090;branch=z9hG4bK831a.2bb3de85.0, \
+SIP/2.0/UDP 85.55.55.12:5090;branch=z9hG4bK831a.2bb3de85.0, \
+SIP/2.0/UDP 85.55.55.12:5060;branch=z9hG4bK831a.2bb3de87.0, \
+SIP/2.0/UDP 85.55.55.12:4060;branch=z9hG4bK831a.2bb3de86.0, \
+SIP/2.0/UDP 85.55.55.12:4050;branch=z9hG4bK831a.2bb3de86.0", get_header(data, "Via:", true));
+
+    EXPECT_STREQ("Record-Route: <sip:85.55.55.12:5090;r2=on;lr>, \
+<sip:10.231.33.44;r2=on;lr>, \
+<sip:10.231.33.77;lr=on>", get_header(data, "Record-Route:", false));
+
+    EXPECT_STREQ("<sip:85.55.55.12:5090;r2=on;lr>, \
+<sip:10.231.33.44;r2=on;lr>, \
+<sip:10.231.33.77;lr=on>", get_header(data, "Record-Route:", true));
+
+    EXPECT_STREQ("<sip:12999999999@85.55.55.12;did=a19.a2e590e>", get_header(data, "Contact:", true));
+}
+
+TEST(Parser, get_header_last) {
+    const char* data = "INVITE sip:3136455552@85.12.1.1:5065 SIP/2.0\r\n\
+From: SIP/2.0/UDP 85.55.55.12:6090;branch=z9hG4bK831a.2bb3de85.0\r\n\
+\r\n\
+";
+    EXPECT_STREQ("", get_header(data, "Via:", false));
+}
+
 TEST(Parser, get_peer_tag__notag) {
     EXPECT_STREQ(NULL, get_peer_tag("...\r\nTo: <abc>\r\n;tag=notag\r\n\r\n"));
 }
@@ -624,6 +728,11 @@ TEST(Parser, get_call_id_short_2) {
     EXPECT_STREQ("testshort2 \t ", get_call_id("...\r\nI:\r\n \r\n \t testshort2 \t \r\n\r\n"));
 }
 
+TEST(Parser, get_short_header_via) {
+    EXPECT_STREQ("\nv:", internal_compact_header_name("Via:"));
+    EXPECT_STREQ("\nm:", internal_compact_header_name("Contact:"));
+}
+
 /* The 3pcc-A script sends "invalid" SIP that is parsed by this
  * sip_parser.  We must accept headers without any leading request/
  * response line:
