@@ -40,6 +40,8 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
+#include <atomic>
+#include <vector>
 
 #ifdef __APPLE__
 /* Provide OSX version of extern char **environ; */
@@ -63,6 +65,7 @@ extern SIPpSocket *stdin_socket;
 
 /* These could be local to main, but for the option processing table. */
 static int argiFileName;
+static std::atomic<bool> run_echo_thread(true);
 
 /***************** Option Handling Table *****************/
 struct sipp_option {
@@ -173,7 +176,7 @@ struct sipp_option options_table[] = {
     {"rsa", "Set the remote sending address to host:port for sending the messages.", SIPP_OPTION_RSA, NULL, 1},
 
 #ifdef USE_TLS
-    {"tls_cert", "Set the name for TLS Certificate file. Default is 'cacert.pem", SIPP_OPTION_STRING, &tls_cert_name, 1},
+    {"tls_cert", "Set the name for TLS Certificate file. Default is 'cacert.pem'", SIPP_OPTION_STRING, &tls_cert_name, 1},
     {"tls_key", "Set the name for TLS Private Key file. Default is 'cakey.pem'", SIPP_OPTION_STRING, &tls_key_name, 1},
     {"tls_ca", "Set the name for TLS CA file. If not specified, X509 verification is not activated.", SIPP_OPTION_STRING, &tls_ca_name, 1},
     {"tls_crl", "Set the name for Certificate Revocation List file. If not specified, X509 CRL is not activated.", SIPP_OPTION_STRING, &tls_crl_name, 1},
@@ -240,6 +243,7 @@ struct sipp_option options_table[] = {
      "- bye\tSend byes for aborted calls\n"
      "- abortunexp\tAbort calls on unexpected messages\n"
      "- pingreply\tReply to ping requests\n"
+     "- cseq\tCheck CSeq of ACKs\n"
      "If a behavior is prefaced with a -, then it is turned off.  Example: all,-bye\n",
      SIPP_OPTION_DEFAULTS, &default_behaviors, 1},
     {"nd", "No Default. Disable all default behavior of SIPp which are the following:\n"
@@ -248,6 +252,7 @@ struct sipp_option options_table[] = {
      "- On unexpected BYE send a 200 OK and close the call\n"
      "- On unexpected CANCEL send a 200 OK and close the call\n"
      "- On unexpected PING send a 200 OK and continue the call\n"
+     "- On unexpected ACK CSeq do nothing\n"
      "- On any other unexpected message, abort the call by sending a BYE or a CANCEL\n",
      SIPP_OPTION_UNSETFLAG, &default_behaviors, 1},
     {"pause_msg_ign", "Ignore the messages received during a pause defined in the scenario ", SIPP_OPTION_SETFLAG, &pause_msg_ign, 1},
@@ -266,16 +271,21 @@ struct sipp_option options_table[] = {
 
     {"", "RTP behaviour options:", SIPP_HELP_TEXT_HEADER, NULL, 0},
     {"mi", "Set the local media IP address (default: local primary host IP address)", SIPP_OPTION_IP, media_ip, 1},
-    {"rtp_echo", "Enable RTP echo. RTP/UDP packets received on port defined by -mp are echoed to their sender.\n"
+    {"rtp_echo", "Enable RTP echo. RTP/UDP packets received on media port are echoed to their sender.\n"
      "RTP/UDP packets coming on this port + 2 are also echoed to their sender (used for sound and video echo).",
      SIPP_OPTION_SETFLAG, &rtp_echo_enabled, 1},
     {"mb", "Set the RTP echo buffer size (default: 2048).", SIPP_OPTION_INT, &media_bufsize, 1},
-    {"mp", "Set the local RTP echo port number. Default is 6000.", SIPP_OPTION_INT, &user_media_port, 1},
-#ifdef RTP_STREAM
+    {"min_rtp_port", "Minimum port number for RTP socket range.", SIPP_OPTION_INT, &min_rtp_port, 1},
+    {"max_rtp_port", "Maximum port number for RTP socket range.", SIPP_OPTION_INT, &max_rtp_port, 1},
     {"rtp_payload", "RTP default payload type.", SIPP_OPTION_INT, &rtp_default_payload, 1},
     {"rtp_threadtasks", "RTP number of playback tasks per thread.", SIPP_OPTION_INT, &rtp_tasks_per_thread, 1},
     {"rtp_buffsize", "Set the rtp socket send/receive buffer size.", SIPP_OPTION_INT, &rtp_buffsize, 1},
-#endif
+    {"rtpcheck_debug", "Write RTP check debug information to file", SIPP_OPTION_SETFLAG, &rtpcheck_debug, 1},
+#ifdef USE_TLS
+    {"srtpcheck_debug", "Write SRTP check debug information to file", SIPP_OPTION_SETFLAG, &srtpcheck_debug, 1},
+#endif // USE_TLS
+    {"audiotolerance", "Audio error tolerance for RTP checks (0.0-1.0) -- default: 1.0", SIPP_OPTION_FLOAT, &audiotolerance, 1},
+    {"videotolerance", "Video error tolerance for RTP checks (0.0-1.0) -- default: 1.0", SIPP_OPTION_FLOAT, &videotolerance, 1},
 
     {"", "Call rate options:", SIPP_HELP_TEXT_HEADER, NULL, 0},
     {"r", "Set the call rate (in calls per seconds).  This value can be"
@@ -449,13 +459,13 @@ void timeout_alarm(int /*param*/)
 
 /* Send loop & trafic generation*/
 
-static void traffic_thread()
+static void traffic_thread(int &rtp_errors, int &echo_errors)
 {
     /* create the file */
     char L_file_name[MAX_PATH];
     sprintf(L_file_name, "%s_%ld_screen.log", scenario_file, (long) getpid());
 
-    getmilliseconds();
+    update_clock_tick();
 
     /* Arm the global timer if needed */
     if (global_timeout > 0) {
@@ -470,7 +480,7 @@ static void traffic_thread()
 
     while (1) {
         scheduling_loops++;
-        getmilliseconds();
+        update_clock_tick();
 
         if (signalDump) {
             /* Screen dumping in a file */
@@ -507,9 +517,9 @@ static void traffic_thread()
             if (!main_scenario->stats->GetStat(CStat::CPT_C_CurrentCall)) {
                 /* We can have calls that do not count towards our open-call count (e.g., dead calls). */
                 abort_all_tasks();
-#ifdef RTP_STREAM
-                rtpstream_shutdown();
-#endif
+                rtp_errors = rtpstream_shutdown(main_scenario->fetchRtpTaskThreadIDs());
+                echo_errors = main_scenario->stats->getRtpEchoErrors();
+
                 /* Reverse order shutdown, because deleting reorders the
                  * sockets list. */
                 for (int i = pollnfds - 1; i >= 0; --i) {
@@ -530,7 +540,7 @@ static void traffic_thread()
             }
         }
 
-        getmilliseconds();
+        update_clock_tick();
 
         /* Schedule all pending calls and process their timers */
         task_list *running_tasks;
@@ -583,8 +593,7 @@ static void traffic_thread()
             sockets_pending_reset.erase(sockets_pending_reset.begin());
         }
 
-        /* Update the clock. */
-        getmilliseconds();
+        update_clock_tick();
         /* Receive incoming messages */
         SIPpSocket::pollset_process(running_tasks->empty());
     }
@@ -596,10 +605,12 @@ static void traffic_thread()
 
 static void rtp_echo_thread(void* param)
 {
-    char* msg = (char*)alloca(media_bufsize);
-    size_t nr, ns;
+    std::vector<char> msg;
+    msg.resize(media_bufsize);
+    ssize_t nr, ns;
     sipp_socklen_t len;
     struct sockaddr_storage remote_rtp_addr;
+    int sock = *(int *)param;
 
 
     int                   rc;
@@ -611,23 +622,30 @@ static void rtp_echo_thread(void* param)
         return;
     }
 
-    for (;;) {
+    // timeout after 100ms, to enable graceful termination of the thread
+    struct timeval tv = {0, 100000};
+    if ((setsockopt(sock, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv)) < 0) ||
+        (setsockopt(sock, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv)) < 0)) {
+        WARNING("Cannot set socket timeout. error: %d", errno);
+    }
+
+    while (run_echo_thread.load(std::memory_order_relaxed)) {
         len = sizeof(remote_rtp_addr);
-        nr = recvfrom(*(int*)param, msg, media_bufsize, 0,
+        nr = recvfrom(sock, msg.data(), media_bufsize, 0,
                       (sockaddr*)&remote_rtp_addr, &len);
 
-        if (((long)nr) < 0) {
+        if (nr < 0) {
+            if (errno == EAGAIN || errno == EWOULDBLOCK)
+                continue;
             WARNING("%s %i",
                     "Error on RTP echo reception - stopping echo - errno=",
                     errno);
             return;
         }
-#ifdef RTP_STREAM
         if (!rtp_echo_state) {
             continue;
         }
-#endif
-        ns = sendto(*(int*)param, msg, nr, 0,
+        ns = sendto(sock, msg.data(), nr, 0,
                     (sockaddr*)&remote_rtp_addr, len);
 
         if (ns != nr) {
@@ -879,6 +897,7 @@ static void help()
         "    1: At least one call failed\n"
         "   97: Exit on internal command. Calls may have been processed\n"
         "   99: Normal exit without calls processed\n"
+        "  253: RTP validation failure\n"
         "   -1: Fatal error\n"
         "   -2: Fatal error binding a socket\n");
 
@@ -991,7 +1010,7 @@ static void print_last_stats()
 {
     interrupt = 1;
     if (sp) {
-      sp->print_closing_stats();
+        sp->print_closing_stats();
     }
     if (main_scenario) {
         stattask::report();
@@ -1064,7 +1083,7 @@ static void releaseGlobalAllocations()
     delete globalVariables;
 }
 
-void sipp_exit(int rc)
+void sipp_exit(int rc, int rtp_errors, int echo_errors)
 {
     unsigned long counter_value_failed = 0;
     unsigned long counter_value_success = 0;
@@ -1083,6 +1102,10 @@ void sipp_exit(int rc)
     screen_exit();
     print_last_stats();
     print_errors();
+    if (sp) {
+        delete sp;
+        sp = NULL;
+    }
 
     /* Close open files. */
     struct logfile_info** logfile_ptr;
@@ -1113,21 +1136,28 @@ void sipp_exit(int rc)
         // successful or not. In order to compute the return code, get
         // the counter of failed calls. If there is 0 failed calls,
         // then everything is OK!
-        if (counter_value_failed == 0) {
-            if (timeout_exit && counter_value_success < 1) {
-                exit(EXIT_TEST_RES_INTERNAL);
+        if ((rtp_errors > 0) || (echo_errors > 0))
+        {
+            exit(EXIT_RTPCHECK_FAILED);
+        }
+        else
+        {
+           if (counter_value_failed == 0) {
+                if ((timeout_exit) && (counter_value_success < 1)) {
+                    exit (EXIT_TEST_RES_INTERNAL);
+                } else {
+                    exit(EXIT_TEST_OK);
+                }
             } else {
-                exit(EXIT_TEST_OK);
+                exit(EXIT_TEST_FAILED);
             }
-        } else {
-            exit(EXIT_TEST_FAILED);
         }
     }
 }
 
 static void sipp_sighandler(int signum)
 {
-    sipp_exit(EXIT_TEST_RES_UNKNOWN);
+    sipp_exit(EXIT_TEST_RES_UNKNOWN, 0, 0);
 }
 
 static void sighandle_set()
@@ -1165,17 +1195,12 @@ static void set_scenario(const char* name)
     }
 }
 
-/**
- * Create and bind media_socket_audio, media_socket_video for RTP and
- * RCTP on try_port and try_port+2.
- *
- * Sets: media_socket_audio and media_socket_audio.
- */
-static int bind_rtp_sockets(struct sockaddr_storage* media_sa, int try_port, int last_attempt)
+static int create_socket(struct sockaddr_storage* media_sa, int try_port, bool last_attempt,
+                         const char *type)
 {
-    /* Create RTP sockets for audio and video. */
-    if ((media_socket_audio = socket(media_sa->ss_family, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
-        ERROR_NO("Unable to create the audio RTP socket");
+    int s = socket(media_sa->ss_family, SOCK_DGRAM, IPPROTO_UDP);
+    if (s == -1) {
+        ERROR_NO("Unable to create the %s RTP socket", type);
     }
 
     if (media_sa->ss_family == AF_INET) {
@@ -1184,35 +1209,36 @@ static int bind_rtp_sockets(struct sockaddr_storage* media_sa, int try_port, int
         (_RCAST(struct sockaddr_in6*, media_sa))->sin6_port = htons(try_port);
     }
 
-    if (::bind(media_socket_audio, (sockaddr*)media_sa, socklen_from_addr(media_sa)) != 0) {
+    if (::bind(s, (sockaddr*)media_sa, socklen_from_addr(media_sa)) != 0) {
         if (last_attempt) {
-            ERROR_NO("Unable to bind audio RTP socket (IP=%s, port=%d)", media_ip, try_port);
+            ERROR_NO("Unable to bind %s RTP socket (IP=%s, port=%d)", type, media_ip, try_port);
         }
-        ::close(media_socket_audio);
-        media_socket_audio = -1;
+        ::close(s);
+        return -1;
+    }
+    return s;
+}
+
+/**
+ * Create and bind media_socket_audio, media_socket_video for RTP and
+ * RCTP on try_port and try_port+2.
+ *
+ * Sets: media_socket_audio and media_socket_video.
+ */
+static int bind_rtp_sockets(struct sockaddr_storage* media_sa, int try_port, bool last_attempt)
+{
+    /* Create RTP sockets for audio and video. */
+    media_socket_audio = create_socket(media_sa, try_port, last_attempt, "audio");
+    if (media_socket_audio == -1) {
         return -1;
     }
 
     /* Create and bind the second/video socket to try_port+2 */
     /* (+1 is reserved for RTCP) */
-    if ((media_socket_video = socket(media_sa->ss_family, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
-        ERROR_NO("Unable to create the video RTP socket");
-    }
-
-    if (media_sa->ss_family == AF_INET) {
-        (_RCAST(struct sockaddr_in*, media_sa))->sin_port = htons(try_port + 2);
-    } else {
-        (_RCAST(struct sockaddr_in6*, media_sa))->sin6_port = htons(try_port + 2);
-    }
-
-    if (::bind(media_socket_video, (sockaddr*)media_sa, socklen_from_addr(media_sa)) != 0) {
-        if (last_attempt) {
-            ERROR_NO("Unable to bind video RTP socket (IP=%s, port=%d)",
-                     media_ip, try_port + 2);
-        }
+    media_socket_video = create_socket(media_sa, try_port + 2, last_attempt, "video");
+    if (media_socket_video == -1) {
         ::close(media_socket_audio);
-        ::close(media_socket_video);
-        media_socket_audio = media_socket_video = -1;
+        media_socket_audio = -1;
         return -1;
     }
 
@@ -1230,15 +1256,27 @@ static void setup_media_sockets()
     struct addrinfo hints = {0,};
     struct addrinfo* local_addr;
     struct sockaddr_storage media_sockaddr = {0,};
+    int try_counter = 0;
+    int max_tries = (min_rtp_port < (max_rtp_port - 2)) ? 100 : 1;
 
-    hints.ai_flags  = AI_PASSIVE;
-    hints.ai_family = PF_UNSPEC; /* use local_ip_is_ipv6 as hint? */
+    media_port = min_rtp_port;
 
+    // [JLTAG]
+    //
+    // RTPCHECK functionality needs port binding to happen only when rtp echo is in use
+    // However since the refactoring in commit "99e847e2a129b5e4c4ccfdd502f79a029929ceb9"
+    // was done media_ip needs to be set unconditionally so I have moved the media_ip
+    // strcpy() to happen outside of the if-block...
+    //
     /* Defaults for media sockets */
     if (media_ip[0] == '\0') {
         strcpy(media_ip, local_ip);
     }
+
     // assert that an IPv6 'media_ip' is not surrounded by brackets?
+    //
+    hints.ai_flags  = AI_PASSIVE;
+    hints.ai_family = PF_UNSPEC; /* use local_ip_is_ipv6 as hint? */
 
     /* Resolving local IP */
     if (getaddrinfo(media_ip,
@@ -1248,29 +1286,30 @@ static void setup_media_sockets()
         ERROR("Unknown RTP address '%s'.\n"
               "Use 'sipp -h' for details", media_ip);
     }
-    memcpy(&media_sockaddr, local_addr->ai_addr,
-           socklen_from_addr(_RCAST(struct sockaddr_storage*, local_addr->ai_addr)));
+    memcpy(&media_sockaddr, local_addr->ai_addr, socklen_from_addr(_RCAST(struct sockaddr_storage*, local_addr->ai_addr)));
     freeaddrinfo(local_addr);
 
     media_ip_is_ipv6 = (media_sockaddr.ss_family == AF_INET6);
 
-    int try_counter;
-    int max_tries = user_media_port ? 1 : 100;
-    media_port = user_media_port ? user_media_port : DEFAULT_MEDIA_PORT;
-    media_socket_audio = media_socket_video = -1;
-    for (try_counter = 1; try_counter <= max_tries; try_counter++) {
-        int last_attempt = (try_counter == max_tries);
+    media_socket_audio = -1;
+    media_socket_video = -1;
 
-        if (bind_rtp_sockets(&media_sockaddr, media_port, last_attempt) == 0) {
-            break;
+    if (rtp_echo_enabled) {
+        for (try_counter = 1; try_counter <= max_tries; try_counter++) {
+            const bool last_attempt = (
+                try_counter == max_tries || media_port >= (max_rtp_port - 2));
+
+            if (bind_rtp_sockets(&media_sockaddr, media_port, last_attempt) == 0) {
+                break;
+            }
+
+            // Old RFC 3551 says:
+            // > RTP data SHOULD be carried on an even UDP port number and
+            // > the corresponding RTCP packets SHOULD be carried on the
+            // > next higher (odd) port number.
+            // So, try only even numbers.
+            media_port += 2;
         }
-
-        // Old RFC 3551 says:
-        // > RTP data SHOULD be carried on an even UDP port number and
-        // > the corresponding RTCP packets SHOULD be carried on the
-        // > next higher (odd) port number.
-        // So, try only even numbers.
-        media_port += 2;
     }
 }
 
@@ -1281,6 +1320,11 @@ int main(int argc, char *argv[])
     pthread_t pthread2_id = 0, pthread3_id = 0;
     unsigned int         generic_count = 0;
     bool                 slave_masterSet = false;
+    int rtp_errors;
+    int echo_errors;
+
+    rtp_errors = 0;
+    echo_errors = 0;
 
     generic[0] = NULL;
 
@@ -1336,6 +1380,9 @@ int main(int argc, char *argv[])
             struct sipp_option *option = find_option(argv[argi]);
             if (!option) {
                 if (argv[argi][0] != '-') {
+                    if ((pass == 0) && (remote_host[0] != 0)) {
+                        ERROR("remote_host given multiple times on command-line (%s and %s)", remote_host, argv[argi]);
+                    }
                     strncpy(remote_host, argv[argi], sizeof(remote_host) - 1);
                     continue;
                 }
@@ -1364,9 +1411,6 @@ int main(int argc, char *argv[])
 #endif
 #ifdef PCAPPLAY
                        "-PCAP"
-#endif
-#ifdef RTP_STREAM
-                       "-RTPSTREAM"
 #endif
                        );
 
@@ -1515,11 +1559,11 @@ int main(int argc, char *argv[])
 #ifdef USE_TLS
                     transport = T_TLS;
                     if (TLS_init() != 1) {
-                        printf("OpenSSL Initialization problem\n");
+                        printf("TLS initialization problem\n");
                         exit(-1);
                     }
 #else
-                    ERROR("To use a TLS transport you must compile SIPp with OpenSSL");
+                    ERROR("To use TLS transport you must compile SIPp with OpenSSL or WolfSSL");
 #endif
                     break;
                 case 'c':
@@ -1541,7 +1585,6 @@ int main(int argc, char *argv[])
                 case 'i':
                     multisocket = 1;
                     peripsocket = 1;
-                    socket_close = false;
                     break;
                 }
 
@@ -1555,13 +1598,12 @@ int main(int argc, char *argv[])
                 break;
             case SIPP_OPTION_NEED_SSL:
                 CHECK_PASS();
-                ERROR("OpenSSL is required for the %s option.", argv[argi]);
+                ERROR("TLS support is required for the %s option.", argv[argi]);
                 break;
             case SIPP_OPTION_MAX_SOCKET:
                 REQUIRE_ARG();
                 CHECK_PASS();
                 max_multi_socket = get_long(argv[argi], argv[argi - 1]);
-                maxSocketPresent = true;
                 break;
             case SIPP_OPTION_CSEQ:
                 REQUIRE_ARG();
@@ -1780,6 +1822,8 @@ int main(int argc, char *argv[])
                             mask = DEFAULT_BEHAVIOR_ABORTUNEXP;
                         } else if (!strcmp(p, "pingreply")) {
                             mask = DEFAULT_BEHAVIOR_PINGREPLY;
+                        } else if (!strcmp(p, "cseq")) {
+                            mask = DEFAULT_BEHAVIOR_BADCSEQ;
                         } else {
                             ERROR("Unknown default behavior: '%s'", token);
                         }
@@ -2078,8 +2122,9 @@ int main(int argc, char *argv[])
     /* to avoid ICMP errors from us. */
     setup_media_sockets();
 
-    /* Creating the remote control socket thread. */
+    /* Creating the remote control socket thread */
     setup_ctrl_socket();
+
     if (!nostdin) {
         setup_stdin_socket();
     }
@@ -2099,15 +2144,10 @@ int main(int argc, char *argv[])
         }
     }
 
-    traffic_thread();
+    traffic_thread(rtp_errors, echo_errors);
 
     /* Cancel and join other threads. */
-    if (pthread2_id) {
-        pthread_cancel(pthread2_id);
-    }
-    if (pthread3_id) {
-        pthread_cancel(pthread3_id);
-    }
+    run_echo_thread.store(false, std::memory_order_relaxed);
     if (pthread2_id) {
         pthread_join(pthread2_id, NULL);
     }
@@ -2122,5 +2162,5 @@ int main(int argc, char *argv[])
 
     free(scenario_file);
     free(scenario_path);
-    sipp_exit(EXIT_TEST_RES_UNKNOWN);
+    sipp_exit(EXIT_TEST_RES_UNKNOWN, rtp_errors, echo_errors); // MAIN EXIT PATH HERE...);
 }
