@@ -35,10 +35,6 @@
  *           Michael Hirschbichler
  */
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE /* needed for strcasestr on cygwin */
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -53,7 +49,6 @@
 #define const_char_nullptr (reinterpret_cast<const char*>(0))
 
 extern bool do_hide;
-extern bool show_index;
 
 SIPpSocket *ctrl_socket = NULL;
 SIPpSocket *stdin_socket = NULL;
@@ -177,7 +172,7 @@ static void process_set(char* what)
     } else if (!strcmp(what, "display")) {
         if (!strcmp(rest, "main")) {
             display_scenario = main_scenario;
-        } else if (!strcmp(rest, "ooc")) {
+        } else if (!strcmp(rest, "ooc") && ooc_scenario) {
             display_scenario = ooc_scenario;
         } else {
             WARNING("Unknown display scenario: %s", rest);
@@ -190,14 +185,6 @@ static void process_set(char* what)
         } else {
             WARNING("Invalid bool: %s", rest);
         }
-    } else if (!strcmp(what, "index")) {
-        if (!strcmp(rest, "true")) {
-            show_index = true;
-        } else if (!strcmp(rest, "false")) {
-            show_index = false;
-        } else {
-            WARNING("Invalid bool: %s", rest);
-        }
     } else {
         WARNING("Unknown set attribute: %s", what);
     }
@@ -883,7 +870,7 @@ int SIPpSocket::empty()
         ret = recvfrom(ss_fd, buffer, readsize, 0, (struct sockaddr *)&socketbuf->addr,  &addrlen);
         break;
     case T_TLS:
-#ifdef USE_OPENSSL
+#if defined(USE_OPENSSL) || defined(USE_WOLFSSL)
         ret = SSL_read(ss_ssl, buffer, readsize);
         /* XXX: Check for clean shutdown. */
 #else
@@ -937,7 +924,7 @@ void SIPpSocket::invalidate()
         return;
     }
 
-#ifdef USE_OPENSSL
+#if defined(USE_OPENSSL) || defined(USE_WOLFSSL)
     if (SSL *ssl = ss_ssl) {
         SSL_set_shutdown(ssl, SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
         SSL_free(ssl);
@@ -1272,10 +1259,13 @@ SIPpSocket::SIPpSocket(bool use_ipv6, int transport, int fd, int accepting):
     /* Initialize all sockets with our destination address. */
     memcpy(&ss_dest, &remote_sockaddr, sizeof(ss_dest));
 
-#ifdef USE_OPENSSL
+#if defined(USE_OPENSSL) || defined(USE_WOLFSSL)
     ss_ssl = NULL;
 
     if (transport == T_TLS) {
+        int flags = fcntl(fd, F_GETFL, 0);
+        fcntl(fd, F_SETFL, flags | O_NONBLOCK);
+
         if ((ss_bio = BIO_new_socket(fd, BIO_NOCLOSE)) == NULL) {
             ERROR("Unable to create BIO object:Problem with BIO_new_socket()");
         }
@@ -1334,7 +1324,7 @@ static int socket_fd(bool use_ipv6, int transport)
 #endif
         break;
     case T_TLS:
-#ifndef USE_OPENSSL
+#ifndef USE_TLS
         ERROR("You do not have TLS support enabled!");
 #endif
     case T_TCP:
@@ -1439,7 +1429,7 @@ SIPpSocket* SIPpSocket::accept() {
     memcpy(&ret->ss_dest, &remote_sockaddr, sizeof(ret->ss_dest));
 
     if (ret->ss_transport == T_TLS) {
-#ifdef USE_OPENSSL
+#if defined(USE_OPENSSL) || defined(USE_WOLFSSL)
         int rc;
         int i = 0;
         while ((rc = SSL_accept(ret->ss_ssl)) < 0) {
@@ -1570,7 +1560,7 @@ int SIPpSocket::connect(struct sockaddr_storage* dest)
     fcntl(ss_fd, F_SETFL, flags);
 
     if (ss_transport == T_TLS) {
-#ifdef USE_OPENSSL
+#if defined(USE_OPENSSL) || defined(USE_WOLFSSL)
         int rc;
         int i = 0;
         while ((rc = SSL_connect(ss_ssl)) < 0) {
@@ -1617,7 +1607,7 @@ int SIPpSocket::reconnect()
     }
 
     if (ss_invalid) {
-#ifdef USE_OPENSSL
+#if defined(USE_OPENSSL) || defined(USE_WOLFSSL)
         ss_ssl = NULL;
 
         if (transport == T_TLS) {
@@ -1873,8 +1863,8 @@ int SIPpSocket::write_error(int ret)
     if ((ss_transport == T_TCP || ss_transport == T_SCTP)
             && errno == EPIPE) {
         nb_net_send_errors++;
-        abort();
         sockets_pending_reset.insert(this);
+        abort();
         if (reconnect_allowed()) {
             WARNING("Broken pipe on TCP connection, remote peer "
                     "probably closed the socket");
@@ -1885,7 +1875,7 @@ int SIPpSocket::write_error(int ret)
         return -1;
     }
 
-#ifdef USE_OPENSSL
+#if defined(USE_OPENSSL) || defined(USE_WOLFSSL)
     if (ss_transport == T_TLS) {
         errstring = SSL_error_string(SSL_get_error(ss_ssl, ret), ret);
     }
@@ -1899,7 +1889,7 @@ int SIPpSocket::write_error(int ret)
 int SIPpSocket::read_error(int ret)
 {
     const char *errstring = strerror(errno);
-#ifdef USE_OPENSSL
+#if defined(USE_OPENSSL) || defined(USE_WOLFSSL)
     if (ss_transport == T_TLS) {
         int err = SSL_get_error(ss_ssl, ret);
         if (err == SSL_ERROR_WANT_READ || err == SSL_ERROR_WANT_WRITE) {
@@ -1963,8 +1953,8 @@ int SIPpSocket::read_error(int ret)
             return 0;
         }
 
-        abort();
         sockets_pending_reset.insert(this);
+        abort();
 
         nb_net_recv_errors++;
         if (reconnect_allowed()) {
@@ -2014,7 +2004,7 @@ void SIPpSocket::buffer_read(struct socketbuf *newbuf)
     prev->next = newbuf;
 }
 
-#ifdef USE_OPENSSL
+#if defined(USE_OPENSSL) || defined(USE_WOLFSSL)
 
 static int send_nowait_tls(SSL* ssl, const void* msg, int len, int /*flags*/)
 {
@@ -2121,7 +2111,7 @@ ssize_t SIPpSocket::write_primitive(const char* buffer, size_t len,
 
     switch(ss_transport) {
     case T_TLS:
-#ifdef USE_OPENSSL
+#if defined(USE_OPENSSL) || defined(USE_WOLFSSL)
         rc = send_nowait_tls(ss_ssl, buffer, len, 0);
 #else
         errno = EOPNOTSUPP;
@@ -2356,9 +2346,9 @@ int open_connections()
             get_inet_address(&remote_sockaddr, remote_ip, sizeof(remote_ip));
             family_hint = remote_sockaddr.ss_family;
             if (remote_sockaddr.ss_family == AF_INET) {
-                strcpy(remote_ip_escaped, remote_ip);
+                strcpy(remote_ip_w_brackets, remote_ip);
             } else {
-                sprintf(remote_ip_escaped, "[%s]", remote_ip);
+                sprintf(remote_ip_w_brackets, "[%.39s]", remote_ip);
             }
             fprintf(stderr, "Done.\n");
         }
@@ -2398,7 +2388,7 @@ int open_connections()
               switch (ret) {
 #ifdef EAI_ADDRFAMILY
                 case EAI_ADDRFAMILY:
-                    ERROR("Network family mismatch for local and remote IP");
+                    ERROR("Network family mismatch for local (%s) and remote (%s, %d) IP", local_ip, remote_ip, family_hint);
                     break;
 #endif
                 default:
@@ -2433,13 +2423,13 @@ int open_connections()
         memcpy(&local_addr_storage, &local_sockaddr, sizeof(local_sockaddr));
 
         if (local_sockaddr.ss_family == AF_INET) {
-            strcpy(local_ip_escaped, local_ip);
+            strcpy(local_ip_w_brackets, local_ip);
             if (!bind_specific) {
                 _RCAST(struct sockaddr_in*, &local_sockaddr)->sin_addr.s_addr = INADDR_ANY;
             }
         } else {
             local_ip_is_ipv6 = true;
-            sprintf(local_ip_escaped, "[%s]", local_ip);
+            sprintf(local_ip_w_brackets, "[%.39s]", local_ip);
             if (!bind_specific) {
                 memcpy(&_RCAST(struct sockaddr_in6*, &local_sockaddr)->sin6_addr, &in6addr_any, sizeof(in6addr_any));
             }
@@ -2579,6 +2569,11 @@ int open_connections()
         }
         sipp_customize_socket(tcp_multiplex);
 
+        /* This fixes local_port keyword value when transport are TCP|TLS and it's defined by user with "-p" */
+        if (sipp_bind_socket(tcp_multiplex, &local_sockaddr, NULL)) {
+            ERROR_NO("Unable to bind TCP socket");
+        }
+
         if (tcp_multiplex->connect(&remote_sockaddr)) {
             if (reset_number > 0) {
                 WARNING("Failed to reconnect");
@@ -2829,7 +2824,7 @@ void SIPpSocket::pollset_process(int wait)
 
     /* We need to process any messages that we have left over. */
     while (pending_messages && loops > 0) {
-        getmilliseconds();
+        update_clock_tick();
         if (sockets[read_index]->ss_msglen) {
             struct sockaddr_storage src;
             char msg[SIPP_MAX_MSG_SIZE];
@@ -2982,7 +2977,7 @@ void SIPpSocket::pollset_process(int wait)
 
 #ifdef HAVE_EPOLL
         unsigned old_pollnfds = pollnfds;
-        getmilliseconds();
+        update_clock_tick();
         /* Keep processing messages until this socket is freed (changing the number of file descriptors) or we run out of messages. */
         while ((pollnfds == old_pollnfds) &&
                 (sock->message_ready())) {
@@ -3022,7 +3017,7 @@ void SIPpSocket::pollset_process(int wait)
 
     /* We need to process any new messages that we read. */
     while (pending_messages && (loops > 0)) {
-        getmilliseconds();
+        update_clock_tick();
 
         if (sockets[read_index]->ss_msglen) {
             char msg[SIPP_MAX_MSG_SIZE];
